@@ -2,10 +2,10 @@ from kgcnn.data.datasets.MoleculeNetDataset2018 import MoleculeNetDataset2018
 
 
 class LipopDataset(MoleculeNetDataset2018):
-    """Store and process full ESOL dataset."""
+    """Store and process full LipopDataset dataset."""
 
     def __init__(self, reload=False, verbose=1):
-        r"""Initialize ESOL dataset.
+        r"""Initialize Lipop dataset.
 
         Args:
             reload (bool): Whether to reload the data and make new dataset. Default is False.
@@ -14,5 +14,5 @@ class LipopDataset(MoleculeNetDataset2018):
         super(LipopDataset, self).__init__("Lipop", reload=reload, verbose=verbose)
 
 
-# data = LipopDataset()
-# data.define_attributes()
+# data = LipopDataset(reload=True)
+
