@@ -1,14 +1,13 @@
 import os
 import numpy as np
-import logging
-
-from kgcnn.mol.methods import ExtensiveMolecularScaler
+from typing import Union
+from kgcnn.scaler.mol import QMGraphLabelScaler
 from sklearn.preprocessing import StandardScaler
 from kgcnn.data.base import MemoryGraphDataset
 from kgcnn.mol.io import parse_list_to_xyz_str, read_xyz_file, \
-    write_mol_block_list_to_sdf, parse_mol_str, read_mol_list_from_sdf_file, write_list_to_xyz_file
-from kgcnn.mol.graphBabel import convert_xyz_to_mol_openbabel, MolecularGraphOpenBabel
-from kgcnn.utils.data import pandas_data_frame_columns_to_numpy
+    write_mol_block_list_to_sdf, read_mol_list_from_sdf_file, write_list_to_xyz_file
+from kgcnn.mol.module_babel import convert_xyz_to_mol_openbabel, MolecularGraphOpenBabel
+from kgcnn.data.utils import pandas_data_frame_columns_to_numpy
 from kgcnn.mol.methods import global_proton_dict, inverse_global_proton_dict
 
 
@@ -120,7 +119,7 @@ class QMDataset(MemoryGraphDataset):
             self.info("Reading single xyz-file.")
             xyz_list = read_xyz_file(self.file_path_xyz)
 
-        # Additionally try to make SDF file
+        # Additionally, try to make SDF file
         if make_sdf:
             self.info("Converting xyz to mol information.")
             mb = self._make_mol_list(xyz_list)
@@ -137,7 +136,7 @@ class QMDataset(MemoryGraphDataset):
         """Try to determine a file name for the mol information to store."""
         return os.path.splitext(self.file_path)[0] + ".xyz"
 
-    def read_in_memory(self, label_column_name: str = None):
+    def read_in_memory(self, label_column_name: Union[str, list] = None):
         """Read xyz-file geometric information into memory. Optionally read also mol information. And try to find CSV
         file with graph labels if a column is specified by :obj:`label_column_name`.
 
@@ -187,8 +186,7 @@ class QMDataset(MemoryGraphDataset):
         edge_attr = []
         for x in mol_list:
             # Must not change number of atoms or coordinates here.
-            mol = MolecularGraphOpenBabel(add_hydrogen=False, make_conformer=False,
-                                          optimize_conformer=False).from_mol_block(x)
+            mol = MolecularGraphOpenBabel().from_mol_block(x, keep_hs=True)
             if mol is None:
                 bond_number.append(None)
                 edge_index.append(None)
@@ -201,127 +199,3 @@ class QMDataset(MemoryGraphDataset):
         self.assign_property("edge_indices", edge_index)
         self.assign_property("edge_number", bond_number)
         return self
-
-
-class QMGraphLabelScaler:
-    """A scaler that scales QM targets differently. For now, the main difference is that intensive and extensive
-    properties are scaled differently. In principle, also dipole, polarizability or rotational constants
-    could to be standardized differently.
-
-    """
-
-    def __init__(self, scaler: list):
-        if not isinstance(scaler, list):
-            raise TypeError("Scaler information for `QMGraphLabelScaler` must be list, got %s." % scaler)
-
-        self.scaler_list = []
-        for x in scaler:
-            # If x is already a scaler, add it directly to the scaler list.
-            if hasattr(x, "fit") and hasattr(x, "transform") and hasattr(x, "inverse_transform"):
-                self.scaler_list.append(x)
-                continue
-            # Otherwise must be serialized version of a scaler.
-            if not isinstance(x, dict):
-                raise TypeError("Single scaler for `QMGraphLabelScaler` must be dict, got %s." % x)
-
-            if "class_name" not in x:
-                raise ValueError("Scaler class for single target must be defined, got %s" % x)
-
-            if x["class_name"] == "StandardScaler":
-                self.scaler_list.append(StandardScaler(**x["config"]))
-            elif x["class_name"] == "ExtensiveMolecularScaler":
-                self.scaler_list.append(ExtensiveMolecularScaler(**x["config"]))
-            else:
-                raise ValueError("Unsupported scaler %s" % x["name"])
-
-        self.scale_ = None
-
-    def _input_for_each_scaler_type(self, scaler, graph_labels, node_number):
-        if isinstance(scaler, StandardScaler):
-            return [graph_labels]
-        elif isinstance(scaler, ExtensiveMolecularScaler):
-            return node_number, graph_labels
-        raise TypeError("Unsupported scaler %s" % scaler)
-
-    def _scale_for_each_scaler_type(self, scaler):
-        if isinstance(scaler, StandardScaler):
-            return scaler.scale_
-        elif isinstance(scaler, ExtensiveMolecularScaler):
-            return scaler.scale_[0]
-        raise TypeError("Unsupported scaler %s" % scaler)
-
-    def fit_transform(self, graph_labels, node_number):
-        r"""Fit and transform all target labels for QM9.
-
-        Args:
-            graph_labels (np.ndarray): Array of QM9 labels of shape `(N, 15)`.
-            node_number (list): List of atomic numbers for each molecule. E.g. `[np.array([6,1,1,1]), ...]`.
-
-        Returns:
-            np.ndarray: Transformed labels of shape `(N, 15)`.
-        """
-        self.fit(graph_labels, node_number)
-        return self.transform(graph_labels, node_number)
-
-    def transform(self, graph_labels, node_number):
-        r"""Transform all target labels for QM. Requires :obj:`fit()` called previously.
-
-        Args:
-            graph_labels (np.ndarray): Array of QM unscaled labels of shape `(N, #labels)`.
-            node_number (list): List of atomic numbers for each molecule. E.g. `[np.array([6,1,1,1]), ...]`.
-
-        Returns:
-            np.ndarray: Transformed labels of shape `(N, #labels)`.
-        """
-        self._check_input(node_number, graph_labels)
-
-        out_labels = []
-        for i, x in enumerate(self.scaler_list):
-            labels = graph_labels[:, i:i+1]
-            out_labels.append(x.transform(*self._input_for_each_scaler_type(x, labels, node_number)))
-
-        out_labels = np.concatenate(out_labels, axis=-1)
-        return out_labels
-
-    def fit(self, graph_labels, node_number):
-        r"""Fit scaling of QM9 graph labels or targets.
-
-        Args:
-            graph_labels (np.ndarray): Array of QM labels of shape `(N, #labels)`.
-            node_number (list): List of atomic numbers for each molecule. E.g. `[np.array([6,1,1,1]), ...]`.
-
-        Returns:
-            self
-        """
-        self._check_input(node_number, graph_labels)
-
-        for i, x in enumerate(self.scaler_list):
-            labels = graph_labels[:, i:i + 1]
-            x.fit(*self._input_for_each_scaler_type(x, labels, node_number))
-
-        self.scale_ = np.concatenate([self._scale_for_each_scaler_type(x) for x in self.scaler_list], axis=0)
-        return self
-
-    def inverse_transform(self, graph_labels, node_number):
-        r"""Back-transform all target labels for QM9.
-
-        Args:
-            graph_labels (np.ndarray): Array of QM scaled labels of shape `(N, #labels)`.
-            node_number (list): List of atomic numbers for each molecule. E.g. `[np.array([6,1,1,1]), ...]`.
-
-        Returns:
-            np.ndarray: Back-transformed labels of shape `(N, 15)`.
-        """
-        self._check_input(node_number, graph_labels)
-
-        out_labels = []
-        for i, x in enumerate(self.scaler_list):
-            labels = graph_labels[:, i:i + 1]
-            out_labels.append(x.inverse_transform(*self._input_for_each_scaler_type(x, labels, node_number)))
-
-        out_labels = np.concatenate(out_labels, axis=-1)
-        return out_labels
-
-    def _check_input(self, node_number, graph_labels):
-        assert len(node_number) == len(graph_labels), "`QMGraphLabelScaler` input length does not match."
-        assert graph_labels.shape[-1] == len(self.scaler_list), "`QMGraphLabelScaler` got wrong number of labels."
